#ifndef COMMUNICATION_H_
#define COMMUNICATION_H_

#include <QObject>
#include <QHttp>
#include <QHash>

class QByteArray;
class ConnectionSettings;

//! Class for handling HTTP requests and responses.
/*!
 *  This class uses the QHttp class to make HTTP requests. HTTP responses are
 *  returned as QByteArray.
 */
class Communication : public QObject
{
	Q_OBJECT

public:
	//! Constructor.
	/*! 
	 *  \param aConnection Reference to ConnectionSettings which holds
	 *  the server to connect to and authentication information.
	*/
	Communication( const ConnectionSettings &aConnection );
	
	virtual ~Communication();
	//! Returns the response of a request identified by aRequestId.
	/*! 
	 *  \param aRequestId Request id number.
	*/
	QByteArray* response( int aRequestId );

	//! Makes a HTTP request to the server defined in the constructor.
	/*! 
	 *  Returns a request id number if successful, otherwise zero.
	 *  \param aCommand Content type string of the HTTP header.
	 *  \param aContent Content body of the HTTP POST request.
	*/
	int request( const QString &aCommand, const QByteArray &aContent );

signals:
	//! Emitted when a request ongoing. Reports the bytes read from the server.
	/*! 
	*   \param aRequestId Request id number.
	*   \param aDone Bytes read from the server.
	*   \param aTotal Bytes total of the response.
	*/
	void readProgress( int aRequestId, int aDone, int aTotal );
	//! Emitted when a request is started.
	/*! 
	*   \param aRequestId Request id number.
	*/
	void requestStarted( int aRequestId );
	//! Emitted when a request is finished.
	/*! 
	*   \param aRequestId Request id number.
	*   \param aError Error code of the request finished.
	*/
	void requestFinished( int aRequestId, int aError );

protected slots:
	//! Connected to QHttp::readyRead to read the response buffer.
	void processResponse( const QHttpResponseHeader& aHeader );
	//! Connected to QHttp::authenticationRequired to provide the user name and password defined in ConnectionSettings
	void handleAuthentication( const QString& aHost, quint16 aPort, QAuthenticator* aAuthenticator );
	//! Connected to QHttp::requestFinished 
	void handleResults( int aId, bool aError );
	//! Connected to QHttp::requestStarted
	void handleRequestStarted( int aRequestId );
	//! Connected to QHttp::dataReadProgress
	void handleReadProgress( int aDone, int aTotal );

private:
	/*!
	 * Instance of Connection settings of the communication
	*/
	ConnectionSettings *iConnectionSettings;
	/*!
	 * Instance of QHttp
	*/
	QHttp* iHttp;
	/*!
	 * QHash of requests and response buffers
	*/
	QHash<int, QByteArray*> iResponses;
	/*!
	 * ID number of a request currently ongoing
	*/
	int iCurrentRequest;
	/*!
	 * Counter how many times auhentication was failed to the server
	*/
	int iAuthFailCount;
};

#endif /*COMMUNICATION_H_*/
