#ifndef WINDOWMANAGER_H_
#define WINDOWMANAGER_H_

#include <QObject>
#include <QTime>
#include "Room.h"
#include "Meeting.h"
#include "PasswordDialog.h"
#include "DeviceManager.h"

class QTimer;
class RoomStatusIndicatorWidget;
class WeeklyViewWidget;
class Engine;
class MeetingInfoDialog;
class SettingsView;
class ProgressBar;
class Configuration;

//! UserInterface class. Behaves as a proxy between the user interface and application's business logic.
/*!
 * UserInterface class. Controls the whole user interface, starting with displaying the appropriate
 * views. It behaves as a proxy between the user interface and application's business logic, it connects
 * the specified components together and forwards the data to the correct place. It also manages the correct
 * appearance of current views on the screen.
 */
class WindowManager : public QObject
{
	Q_OBJECT

public:
	//! Constructor.
	/*!
	 * Constructor of WindowManager.
	 * \param aConfiguration The pointer to configuration.
	 */
	WindowManager( Configuration *aConfiguration );
	//! Destructor.
	virtual ~WindowManager();
	/*!
	 * Displays an error message
	 * \param aErrorMessage Message to be displayd
	 */
	void error( const QString &aErrorMessage );
	//! Updates the rooms status.
	/*! 
	 * Forwards the signal of changed status to current view.
	 * \param aRoom Room which status is changed.
	 * \param aStatus Current status of room.
	 * \param aTime Time when status is changed.
	 */
	void roomStatusChanged( Room *aRoom, Room::Status aStatus, QTime aTime );
	//! Shows the password dialog.
	/*!
	 * Shows the password dialog.
	 * \param aAdminPassword The correct password.
	 * \param aMessage The message to be shown in the password dialog.
	 */
	void showPasswordDialog( QByteArray aAdminPassword, const QString &aMessage );
	//! Closes the password dialog.
	/*!
	 * Closes the password dialog.
	 */
	void closePasswordDialog();
	//! Displays the weekly view.
	/*!
	 * Displays the weekly view.
	 */
	void showWeeklyView();
	//! Displays the meeting info dialog.
	/*!
	 * Displays the meeting info dialog. 
	 * \param aMeeting Meeting to be displayd
	 */
	void showMeetingInfo( Meeting *aMeeting );
	//! Returns the pointer to the weekly view. 
	/*!
	 * Returns the pointer to the weekly view.
	 */
	WeeklyViewWidget * weeklyView();
	//! Switches the views to full screen.
	/*!
	 * Switches the views to full screen.
	 */
	void fullScreen();
	//! Shows the progress bar.
	/*!
	 * Starts showing the progress bar.
	 * \param aText The text to be shown in progress bar.
	 *  \param aCancellable Is the Cancel button visible. By default not visible.
	 */
	void showProgressBar( const QString &aText, bool aCancellable = false );
	//! Closes the progress bar.
	/*!
	 * Closes the progress bar.
	 */
	void closeProgressBar();
	
	void insertMeeting( Meeting *aMeeting );
	
	void deleteMeeting( Meeting *aMeeting );
	
	//! Shows any view specific indicators for connection error
	void connectionLost();
	
	//! Removes any view specific indicators for connection error
	void connectionEstablished();

signals:
	//! Request current status of the room.
	/*!
	 * Signal is emitted when there is need to check current status of room aRoom.
	 * \param aRoom Meetingroom which status is requested.
	 */
	void roomStatusInfoNeeded( Room *aRoom );
	//! Indicate that some user event has happened.
	/*!
	 * Signal is emitted if some user event has happened.
	 */
	void observedEventDetected();
	//! Meeting activated.
	/*!
	 * Signal is emitted when a meeting is clicked by the user.
	 * \param aMeeting actived meeting.
	 */
	void meetingActivated( Meeting *aMeeting );
	//! Signals if the shown week has been changed.
	/*!
	 * Signal. Emitted if the shown week has been changed.
	 * \param aDate The first date of the shown week.
	 */
	void shownWeekChanged( QDate aDate );
	//! Signals change of the meeting room.
	/*!
	 * Signal is emitted when meeting room is changed.
	 * \param aRoom Selected meeting room.
	 */
	void currentRoomChanged( Room *aRoom );
	//! Signals when the password dialog buttons are clicked.
	/*!
	 * Signal is emitted when the password dialog buttons are clicked.
	 * \param aPasswordStatus The status of the password.
	 */
	void passwordEntered( PasswordDialog::PasswordStatus aPasswordStatus );
	//! Signals when the cancel button in the progress bar is clicked.
	/*!
	 * Signal is emitted when the cancel button in the progress bar is clicked.
	 */
	void progressBarCancelled();
	
public slots:
	//! Slot for displaying the screensaver (room status view).
	/*!
	 * Slot. Displays the screensaver.
	 */
	void showRoomStatus();
	//! Slot for updating the time.
	/*!
	 * Slot. Forwards the signal of changed time to current view.
	 * \param aCurrentDateTime Current date and time.
	 */
	void distributeDateTimeInfo( QDateTime aCurrentDateTime );
	
	void updateProgressBar( const QString &aMessage );
	
private slots:
	//! Displays the settings view
	void showSettingsView();

private:
	//! Name of the application.
	QString iApplicationName;
	//! Defines whether the views should be shown as full screen 
	bool iFullScreen;
	//! Pointer to the configuration.
	Configuration *iConfiguration;
	//! Pointer to the weekly view.
	WeeklyViewWidget *iWeeklyView;
	//! Pointer to the screensaver (room status view).
	RoomStatusIndicatorWidget *iRoomStatusView;
	//! Pointer to the meeting info dialog
	MeetingInfoDialog *iMeetingInfo;
	//! Pointer to the settings view
	SettingsView *iSettingsView;
	//! Pointer to the progress bar
	ProgressBar *iProgressBar;
	//! Pointer to the password dialog.
	PasswordDialog *iPasswordDialog;

};

#endif /*WINDOWMANAGER_H_*/
